'' ILI9325 driver using pins P0-P11 for the display and P16-P23 for the touchscreen
'' James Moxham 2011

CON


  _clkmode      = xtal1 + pll16x                        ' use crystal x 16
  _xinfreq      = 5_000_000

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1                                           ' -1 ifnot installed.
  _SD_CD = -1                                           ' -1 ifnot installed.

  _RTC_DAT = -1                                         ' -1 ifnot installed. For kye's sd driver
  _RTC_CLK = -1                                         ' -1 ifnot installed.

  _TV_Enabled = true                                    ' TV display on
  _TV_Pin = 16                                          ' group of 3 pins starting at pin 16 = 16,17,18

  Touch_Clock = 23                                        ' Touch screen pins (shared with VGA and one, penirq, shared with TV)
  Touch_ChipSelect = 22
  Touch_DataIn = 21
  Touch_Busy = 20
  Touch_DataOut = 19
  Touch_Penirq = 18
  
OBJ
 delay:         "timing"                                ' thanks to parallax
 fat:           "SD3.01_FATEngine.spin"                 ' thanks to Kye
 text:          "tv_text"                               ' thanks to Chip
 str:           "ASCII0_STREngine.spin"                 ' thanks to Kye 
 spi:           "SPI_ASM"                               ' thanks to Beau
 ILI9325:       "ILI9325_Dracblade"                     ' J Moxham

VAR

   byte LineOfText1[80] ' general purpose string buffer
   'byte LineOfText2[80] ' general purpose string buffer                                      
  
PUB Main
    text.start(_TV_Pin)                                 ' start the TV driver
    text.clearscreen                                    ' clear the screen
    ILI9325.start_ram                                   ' start the external ram driver in a cog
    ILI9325.start_ILI9325                               ' start the ILI9325 display
     PrintStringCR(string("Testing for SD card"))       ' helpful message if card is out, better than just a blank screen  
    'Touchscreen                                        ' test the touchscreen
    ifnot( fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))   
      reboot
    fat.mountPartition(0)                               ' mount the sd card
    OUTA[Touch_ChipSelect] := 0                         ' enable the touch screen
    SPI.start(3,0)                                      ' delay,state for the touchscreen
    
    'createicon      ' only do once per icon, saves a .icn file to the sd card, takes 8 seconds. Or use vb.net program
    Clearscreen
    RawtoILI ' background .raw file RGB , picture must be 240 *wide* and 320 high ie portrait format 
    DisplayIcons
    PrintStringCR(string("touch the screen")) 
    Touchscreen
    

PUB ReadBackground | ramaddress             ' block move data from sd card to memory, much faster than one byte at a time
    PrintStringCR(string("Block load"))                 ' message on the TV screen
    fat.openfile(string("icons320.ili"),"R")            ' open an image (.ili extension)
    ramaddress := 0
    repeat 600
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card
      ILI9325.docmd("A",@sdbuffer,ramaddress,256)       ' move data from hub to external ram 256 bytes at a time    
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile   
    PrintStringCR(string("Closing file"))
    RamToDisplay


PUB RawToILI | ramaddress                         ' convert 3 byte .raw file to 2 byte .ili file
    'PrintStringCR(string("Load file"))                 ' message on the TV screen    
    fat.openfile(string("beach.raw"),"R")                ' open an image
    ramaddress := 0
    repeat 600
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' move bytes to external ram
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile
    'PrintStringCR(string("Closing file"))
    RamToDisplay                              ' move bytes from ram out to display


PUB CreateIcon | iconaddress,maskaddress,i,j
  ' this creates a .icn file which is 4 bytes per pixel, red, green, blue and alpha. Do once when setting up icons
  ' so one long per pixel. All values are 0-255
  ' takes 8 seconds per icon so don't do this every load
  ' to create an icon using paint shop pro or similar
  '1) Open a .png icon file which has a transparancy layer
  '2) Copy and paste the picture to a new picture and save as a .raw RRRRRRRRGGGGGGGGBBBBBBBB file
  '3) View the mask (in PSP there is a Mask on the top menu bar, then Edit).
  '4) Masks/Save the file, and rename the extension as .bmp.
  '5) Close down all open files in PSP, then File/Open and open the .bmp file above. This should be 256 color grayscale, ie 1 byte per pixel.
  '6) Save that file as a .raw file, eg mask.raw
  '7) Copy the mask.raw file to the sd card This .raw file has 1 byte per pixel (note different to the picture which is 3 bytes per pixel)    

  ' in a family of icons the mask.raw file is going to be the same for all of them
  ' so on the sd card one might have two files - clock.raw and mask.raw
  ' this function combines these together into an .icn file. It is in spin which is slower but this is only ever run once
  PrintStringCR(string("myfile.raw and mask.raw to myfile.icn"))   
  fat.openfile(string("clock.raw"),"R")                 ' open an icon file which is 59x60 bytes x3 = 10620
  iconaddress := 0 
  repeat 10620                                          ' 
    i := fat.readbyte
    ILI9325.docmd("W",@i,iconaddress,1)                  ' write n to ramaddress
    iconaddress +=1                                     ' increment the counter
  fat.closefile
  fat.openfile(string("mask.raw"),"R")                 ' open an icon file which is 59x60 bytes x3 = 10620
  maskaddress := 16384
  repeat 3540                                           ' one byte per pixel in the mask
    i := fat.readbyte
    ILI9325.docmd("W",@i,maskaddress,1)                  ' write n to ramaddress
    maskaddress += 1                                    ' increment the counter
  fat.closefile
  iconaddress := 0                                      ' reset counters
  maskaddress := 16384
                                                        ' see kyedos for using "stringpointer" instead of text 
  result := \fat.newFile(string("clock.icn"))           ' All aborts traps must have "variable := \function" setup.
  if(fat.partitionError == fat#Entry_Already_Exist)     ' file already exists
     result := \fat.deleteEntry(string("clock.icn"))    ' delete the old file
     result := \fat.newfile(string("clock.icn"))        ' recreate the file
  fat.openfile(string("clock.icn"),"W")
  repeat 3540
    j := 0                                              ' clear j
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram into i = the red byte
    iconaddress += 1
    j := j | i                                          ' put the red byte into lsb place
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram
    iconaddress += 1
    j := j | (i << 8)                                   ' put the green byte into place
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram 
    iconaddress += 1
    j := j | (i << 16)                                  ' put the blue byte into place
    ILI9325.docmd("R",@i,maskaddress,1)                 ' read from ram into i the alpha byte
    maskaddress += 1
    j := j | (i << 24)                                  ' put the alpha byte into msb place - hex editor = RGBA
    fat.writelong(j)                                    ' store to SD card file
  fat.closefile
  PrintStringCR(string("clock.icn created"))                                                                           

PUB RamToDisplay ' dumps preloaded ram to display starting address 0
    'Clearscreen                                        ' leave out clearscreen
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in
    ILI9325.docmd("C",0,0,153600)                       ' move bytes from ram out to the display         

PUB Clearscreen | i
    i := 0
    repeat 512
      sdbuffer[i] := 0                         ' array all to zero
      i := i + 1
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in   
    repeat 300
      ILI9325.docmd("D",@sdbuffer,0,512)                       ' move bytes from ram out to the display


PUB TransparentIcon(x,y) | sizex,sizey,backgroundaddress,ramaddress ' draw icon name in LineOfText1
    x := x * 60 ' pass x as 0,1,2,3 from touchscreen convert to a pixel coordinate
    y := y * 60 ' pass y as 0,1,2,3,4 from touchscreen
    sizex := 59
    sizey := 60
    ILI9325.draw(x,y,x+sizex-1,y+sizey-1)                           ' set up the area of the screen to draw in
     ' background is stored in memory 0 to 153599
     ' background behind the icon temp store =64*64*4 = 16384 bytes 153600 to 169984 so contiguous bytes
    backgroundaddress := (((y*240)+x) << 1)              ' calculate the correct location in background ram
    ramaddress := 153600
    repeat 30
      ILI9325.docmd("R",@sdbuffer,backgroundaddress,(sizex << 2)) ' get 59*2 *2 bytes from background. Instead of 60 lines with everything multiplied by 2, do 30 lines and multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' save to a temp buffer 
      backgroundaddress += (240 << 2)                          ' next row = 240 pixels *2  *2
      ramaddress += (sizex << 2)                        ' 59*2  *2
    ramaddress := 169984 ' location to store icon
    'PrintStringCR(string("Load"))            ' message on the TV screen
    fat.openfile(@lineoftext1,"R")              ' open an icon image (.icn extension = RGBA)
    repeat sizey
      fat.readdata(@sdbuffer,(sizex << 2))                 ' sizex pixels * 4 bytes per pixel so multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' now move data from hub to external ram
      ramaddress += (sizex << 2)                        ' 4 bytes per pixel so multiply by 4   
    fat.closefile   
    'PrintStringCR(string("Closing file"))
    ILI9325.docmd("F",153600,169984,(sizex*sizey))  ' background,icon,number of pixels


PUB DisplayIcons
    str.copy(string("clock.icn"),@lineoftext1) ' copy fixed value into a string
    TransparentIcon(1,1) ' display icon in lineoftext1
    str.copy(string("cpm.icn"),@lineoftext1) ' copy fixed value into a string  
    TransparentIcon(2,1) ' display icon in lineoftext1
    str.copy(string("photos.icn"),@lineoftext1) ' copy fixed value into a string
    TransparentIcon(0,2) ' display icon in lineoftext1
    str.copy(string("weather.icn"),@lineoftext1) ' copy fixed value into a string  
    TransparentIcon(2,4) ' display icon in lineoftext1   


PRI TouchScreen  | yval, xval,x,y
' *** note these pins are changing in the next board to DO = 20, CLK =21, DI = 22, CS = 23 (busy and penirq not connected)
' also with penirq on pin 18 this upsets the TV display
'  Touch_Clock = 23                                       ' clock
'  Touch_ChipSelect = 22                                  ' chip select needs to be low to enable the touchscreen
'  Touch_DataIn = 21                                      ' data into the chip = an output from the prop
'  Touch_Busy = 20                                        ' busy ? not needed - just returns /cs
'  Touch_DataOut = 19                                     ' data out of the chip = an input to the prop
'  Touch_Penirq = 18                                      ' pen down (? use for wakeup)
  OUTA[Touch_ChipSelect] := 0                           ' enable the touch screen
  SPI.start(3,0)                                        ' delay,state
  'Shiftout = data,clock,mode,bits,value
  'shiftin = data,clock,mode,bits
  'SPI read S, A2, A1, A0, Mode, SER, PD1, PD0
  '         1  0   0   1    1     0    0    0
  ' MSB first
  ' 001 is X
  ' 101 is Y
    repeat
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1101_0000)  ' reads x from  500 to 3700  (off < 500 )
      xval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1001_0000)  ' reads y from 400 to 3800 (off > 3800 ) 
      yval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      if xval <500                                      ' if less than 500 then not touched, change to -1
        xval := -1                                      ' no touch is -1
      else
        xval := 3700 - xval                             ' move origin bottom right to bottom left
      if yval > 3800                                   ' if >3800 then not touched, change to -1
        yval := -1  
      delay.pause1ms(100)                              ' short delay for the display
      'printdecimal(xval)                               ' display the numbers
      'printdecimal(yval)
      'crlf                                             ' new line
      if (xval <> -1)  and (yval <> -1)
        x := (xval - 500)/800                           ' 0,1,2,3
        y := 4 - ((yval - 400) / 680)                   ' 0,1,2,3,4 and 0,0 is top left
        printdecimal(x)                               ' display the numbers
        printdecimal(y)
        crlf
        if (x == 0) and (y == 0)
            str.copy(string("twitter.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 0)
            str.copy(string("calc.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 0)  
            str.copy(string("calendar.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 0)
            str.copy(string("camera.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 1)  
            str.copy(string("cydia.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 1)
            str.copy(string("facebook.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 1)
            str.copy(string("contacts.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 1)
            str.copy(string("cpm.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 2)  
            str.copy(string("ifile.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 2)
            str.copy(string("itunes.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 2)  
            str.copy(string("phone.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 2)
            str.copy(string("mail.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 3)  
            str.copy(string("maps.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 3)
            str.copy(string("messages.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 3)
            str.copy(string("music.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 3)
            str.copy(string("notes.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 4)  
            str.copy(string("macri.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 4)
            str.copy(string("safari.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 4)  
            str.copy(string("settings.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 4)
            str.copy(string("stocks.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1

        delay.pause1ms(250)' only display once so delay half a second to "debounce"

 
PUB PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PUB PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    if (_TV_Enabled)
      text.str(pstring)
'    if (_VGA_Enabled)  
'      vt100.str(pstring)                                 ' send to vga screen
'    if (_Serial_Enabled)   
'    sio.str(0,pstring)                                 ' send to com port
'    if (_lcd_Enabled)
'       lcd.str(pstring)                                   ' send to 20x4 lCD display

PUB PrintChar(c) ' sends c to vga and serial port
    if (_TV_Enabled)
      text.out(c)
'    if (_VGA_Enabled)
'      vt100.out(c)                                        ' send to vga
'    if (_Serial_Enabled)
'      sio.tx(0,c)                                         ' send to com port
'    if (_lcd_Enabled)
'      LCD.out(c)                                          ' send to LCD display      
PUB CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga

PUB PrintDecimal(number)                                ' print the decimal value, useful for debuggin
     PrintString(str.integerToDecimal(number, 10))

DAT
sdbuffer        byte    $0[512]  ' 512 byte buffer for sd card interface
buffer2         byte    $0[512]  ' 512 general purpose hub buffer